@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 import socket
-from multiprocessing import Process
 from threading import Thread
 from time import sleep
 from wsgiref.simple_server import make_server
@@ -8,7 +7,7 @@ from wsgiref.simple_server import make_server
 import dash
 import dash_core_components as dcc
 import dash_html_components as html
-import psutil
+import pathos
 
 from .Dashboard_Control import _Dashboard_Control
 
@@ -154,16 +153,12 @@ class AgentDashboard:
             if sock.connect_ex((ip_addr, _port)) == 0:
                 sock.shutdown(socket.SHUT_RDWR)
                 return False
-            # So no dashboard seems to running. Here we check, if anything else is
-            # running on that port.
-            for connection in psutil.net_connections():
-                if connection.laddr.port == _port and connection.laddr.ip == ip_addr:
-                    return False
             # Seems as if, we can actually start our dashboard server.
             return True
 
 
-class AgentDashboardProcess(AgentDashboard, Process):
+
+class AgentDashboardProcess(AgentDashboard, pathos.helpers.mp.Process):
     """Represents an agent dashboard for the osBrain backend"""
 
 
