@@ -15,8 +15,8 @@ class MultiGeneratorAgent(AgentMET4FOF):
         if self.current_state == "Running":
             sine_data = self._sine_stream.next_sample()  # dictionary
             cosine_data = self._sine_stream.next_sample()  # dictionary
-            self.send_output(sine_data["x"], channel="sine")
-            self.send_output(cosine_data["x"], channel="cosine")
+            self.send_output(sine_data["quantities"], channel="sine")
+            self.send_output(cosine_data["quantities"], channel="cosine")
 
 
 class MultiOutputMathAgent(AgentMET4FOF):
